@@ -21,6 +21,7 @@ stripTMBVC <- function(x) {
     r <- VarCorr(x)[["cond"]]
     for (i in seq_along(r)) {
         attr(r[[i]],"blockCode") <- NULL
+        class(r[[i]]) <- c("matrix", "array")
     }
     return(r)
 }
@@ -34,9 +35,12 @@ test_that("basic glmer vs glmmTMB", {
 
 ## have to take only last 4 lines
 ## some white space diffs introduced in fancy-corr-printing
+## FIXME: need to match formatting better?
 pfun <- function(x) squash_white(capture.output(print(VarCorr(x),digits=2)))
-expect_equal(tail(pfun(fm1),4),
-             pfun(fm1C))
+test_that("VarCorr output equivalent to lme4", {
+    expect_equal(tail(pfun(fm1),4),
+                 pfun(fm1C))
+})
 
 data("Pixel", package="nlme")
 ## nPix <- nrow(Pixel)
@@ -198,3 +202,43 @@ test_that("VarCorr omits resid when dispformula=~0", {
 test_that("vcov(.,full=TRUE) works for zero-disp models", {
     expect_equal(dim(vcov(fm0,full=TRUE)),c(4,4))
 })
+
+test_that("blockCode set correctly in VarCorr", {
+    set.seed(102)
+    n <- 25                                              ## Number of time points
+    x <- MASS::mvrnorm(mu = rep(0,n),
+                       Sigma = .7 ^ as.matrix(dist(1:n)) )    ## Simulate the process using the MASS package
+    y <- x + rnorm(n, sd = 0.1)                                   ## Add measurement noise
+    times <- factor(1:n, levels=1:n)
+    group <- factor(rep(1,n))
+    dat0 <- data.frame(y, times, group)
+    ## suppress non-pos-def warning
+    suppressWarnings(model <- glmmTMB(y ~ ar1(factor(times) + 0 | group),
+                                      ziformula = ~ ar1(factor(times) + 0 | group),
+                                      data=dat0, family = gaussian())
+                     )
+    expect_equal(attr(VarCorr(model)[["zi"]]$group, "blockCode"), c(ar1=3))
+})
+
+test_that("VarCorr for models with RE in dispersion", {
+    data("sleepstudy", package = "lme4")
+
+    fit <- suppressWarnings(glmmTMB(Reaction ~ Days + (Days | Subject),
+                   dispformula = ~ Days + (Days | Subject),
+                   data = sleepstudy))
+
+    vc <- VarCorr(fit)
+    expect_equal(c(vc$disp$Subject),
+                   c(0.127005710245793, -4.71258573290661e-17, -4.71258573290661e-17, 
+                     1.865081358102e-32),
+                 tolerance = 1e-5)
+})
+
+## need to fix diag, comp symm ...
+skip_models <- character(0)
+models <- setdiff(gt_load("test_data/models.rda"), skip_models)
+for (m in models) {
+    obj <- get(m)
+    if (!inherits(obj, "glmmTMB")) next
+    capture.output(print(VarCorr(obj)))
+}
